#!/bin/sh 

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2005 European Southern Observatory
# LICENSE
#
# NAME:      dfschecklib.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    CG - ESO/DMD/SEG
# DESCRIPTION: Library with functions for running tests
# 	       Functions implemented:
#		 set_echoes()
#		 necho()
#		 myecho()
#		 ask_yn()
#		 check_file_exist ()
#		 check_access_host_account()
#		 check_file_does_not_exist ()
#		 check_directory_exist ()
#		 check_protections ()
#		 check_owner_group ()
#		 find_ccslite_environment()
#		 find_vlt_boot_release ()
#		 check_env_contains_regexpr ()
#		 check_string_contains_regexpr ()
#		 check_file_contains_regexpr ()
#		 check_only_one_process_run ()
#		 check_is_a_link ()
#		 check_is_not_a_link ()
#		 check_soft_link ()
#		 check_process_run ()
#		 check_process_owned_by ()
#		 check_user_exist ()
#		 check_group_exist ()
#		 check_user_belongs_to_group ()
#		 check_command_exist ()
#
# USAGE:     From another scripts, e.g. dfscheck.sh
#              dirname=`dirname $0`
#              library=$dirname/dfschecklib.sh
#              . $library
#              set_echoes
#
# EXAMPLES: See dfscheck.sh as an script using the library

#
# $Id: dfschecklib.sh,v 1.3 2005/11/28 22:33:41 cguirao Exp $
#
# who            when            what
#------------------------------------------------------------------------------
# C. Guirao      31/03/05        created
#------------------------------------------------------------------------------
 

#
# Check how the echo command behaves
#
set_echoes()
{
  if [ "`echo -n a`" = a ]; then
    # BSD echo
    NNN=-n
    CCC=
  else
    # USG echo
    NNN=
    CCC='\c'
  fi
  readonly NNN CCC 
}

#
#  A safe way to perform echo without sending a new-line
#
necho()
{
  if [ "$ECHO" = "1" ]; then
    echo $NNN "$*" $CCC 
  fi
}       

myecho()
{
  if [ "$ECHO" = "1" ]; then
    echo "$*"
  fi
}       

#
#  Get a yes/no answer from the user; $1 is the prompt, $2 the default
#
ask_yn()
{
  prompt="$1"
  def="${2:-y}"           # yes is the default if not supplied
  while true; do
    necho "$prompt [$def]: "
    read answer
    answer="${answer:-$def}"
    case "$answer" in
      [yY])
        answer=y
        break
        ;;
      [nN])
        answer=n
        break
        ;;
      *)
        error Please enter "'y'" or "'n'".
        ;;
    esac
  done
  [ $answer = y ]
}

check_file_exist ()
{
  necho "File $1 "
  if [ -f $1 ]; then
    myecho "exists. (OK)"
    return 0
  else 
    myecho "does not exist. (ERROR)"
    return 1
  fi
}

check_access_host_account()
{
  $RSH $1 -l $2 true >/dev/null 2>&1
  if [ $? = 0 ]; then
    myecho "Have access to host $1 as account $2 (OK)"
  else
    myecho "Have no access to host $1 as account $2 (ERROR)"
  fi
}

check_file_does_not_exist ()
{
  necho "File $1 "
  if [ -f $1 ]; then
    myecho "exists. (ERROR)"
    return 1
  else 
    myecho "does not exist. (OK)"
    return 0
  fi
}

check_directory_exist ()
{
  necho "Directory $1 "
  if [ -d $1 ]; then
    myecho "exists. (OK)"
    return 0
  else 
    myecho "does not exist. (ERROR)"
    return 1
  fi
}

check_protections ()
{
  target=$1
  if [ -h $1 ]; then 
    orig=`ls -ld $1 | awk '{print $9 }'`
    orig=`dirname $orig`
    target=`ls -ld $1 | awk '{print $11}'`
    echo $target | egrep  '^/' > /dev/null
    if [ $? = 1 ] ; then
      target=$orig/$target
    fi
  fi
  protections=`ls -ld $target | awk '{print $1}'`
  if [ "$protections" = "$2" ]; then
    myecho "File/directory $1 has protections $2 (OK)"
    return 0
  else 
    myecho "File/directory $1 has not protections $2 (ERROR)"
    return 1
  fi
}

check_owner_group ()
{
  target=$1
  if [ -h $1 ]; then 
    orig=`ls -ld $1 | awk '{print $9 }'`
    orig=`dirname $orig`
    target=`ls -ld $1 | awk '{print $11}'`
    echo $target | egrep  '^/' >/dev/null
    if [ $? = 1 ] ; then
      target=$orig/$target
    fi
  fi
  owner=`ls -ld $target | awk '{print $3}'`
  group=`ls -ld $target | awk '{print $4}'`
  if [ "$owner" = "$2" -a "$group" = "$3" ]; then
    myecho "UID:GID for $1 is $2:$3 (OK)"
    return 0
  else 
    myecho "UID:GID for $1 is not $2:$3 (ERROR)"
    return 1
  fi
}

find_ccslite_environment()
{
  case $HOSTNAME in
    w*) RTAPENV=$HOSTNAME ;;
    *)  RTAPENV=w$HOSTNAME ;;
  esac
}

find_vlt_boot_release ()
{
  if ! check_file_exist /vlt/System/vltsw_bootstrapping_release ; then 
    return 1
  fi
  necho "Environment VLTSW_RELEASE "
  VLTSW_RELEASE=`egrep -v '^#|^[ 	]*$' /vlt/System/vltsw_bootstrapping_release`
  if [ -z "$VLTSW_RELEASE" ]; then 
    myecho "is empty. (ERROR)"
    return 1 
  else 
    myecho "is not empty. (OK)" 
    return 0
  fi
}

check_env_contains_regexpr ()
{
  necho "Environment variable $3 "
  ECHO=0  
  if check_string_contains_regexpr "$1" "$2" ; then
    ECHO=1  
    myecho "contains regexpr  \"$2\" (OK)" 
  else
    ECHO=1  
    myecho "does not contain regexpr  \"$2\" (ERROR)" 
  fi
}

check_string_contains_regexpr ()
{
  necho "String $1 "
  echo "$1" | egrep  "$2"  >/dev/null
  if [ $? != 0 ] ; then 
    myecho "does not contain regexpr  \"$2\" (ERROR)" 
    return 1
  else
    myecho "contains regexpr  \"$2\" (OK)"
    return 0
  fi
} 

check_file_contains_regexpr ()
{
  necho "File $1 "
  egrep  "$2" "$1" >/dev/null
  if [ $? != 0 ] ; then 
    myecho "does not contains regexpr  \"$2\" (ERROR)" 
    return 1
  else
    myecho "contains regexpr  \"$2\" (OK)"
    return 0
  fi
} 

check_only_one_process_run ()
{
  count=`ps -ef | egrep -v "egrep $1" | egrep "$1" | wc -l`
  if [ $count = 0 ]; then 
    myecho "None process \"$1\" is running. (ERROR)"  
    return 1
  elif [ $count != 1 ]; then
    myecho "More than one process \"$1\" is running. (ERROR)"  
    return 1
  else
    myecho "Only one process \"$1\" is running. (ERROR)"  
    return 0
  fi 
} 

check_is_a_link ()
{
  if [ -h "$1" ]; then
    myecho "$1 is a soft link. (OK)"
    return 0
  else
    myecho "$1 is not a soft link. (ERROR)"
    return 1
  fi
}

check_is_not_a_link ()
{
  if [ -h "$1" ]; then
    myecho "$1 is a soft link. (ERROR)"
    return 1
  else
    myecho "$1 is not a soft link. (OK)"
    return 0
  fi
}

check_soft_link ()
{
  if [ -h "$1" ]; then
    target=`ls -l $1 | awk '{print $11}'`
    if [ "$target" = "$2" ]; then
      myecho "Link $1 is pointing to $2 (OK)"
      return 0
    else
      myecho "Link $1 is not pointing to $2, but to $target (ERROR)"
      return 1
    fi
  else
    myecho "Link $1 is not a soft link. (ERROR)"
    return 1
  fi
}

check_process_run ()
{
  necho "Process \"$1\" "
  ps -ef | sed 's/^................................................//' | egrep "^$1" > /dev/null
  if [ $? != 0 ] ; then 
    myecho "is not running. (ERROR)"  
    return 1
  else
    myecho "is running. (OK)"  
    return 0
  fi 
} 
 
check_process_owned_by ()
{
  if ! check_process_run "$1"  ; then 
    return 1
  fi
  necho "Process \"$1\" "
  ps -ef | awk '{ if ($1 == "'$2'" ) print $0}' | sed 's/^................................................//' | egrep "^$1" > /dev/null
  if [ $? != 0 ] ; then 
    myecho "is not owned by $2. (ERROR)"  
    return 1
  else
    myecho "is owned by $2. (OK)"  
    return 0
  fi 
} 


check_user_exist ()
{
  necho "User $1 "
  egrep  "^${1}:" /etc/passwd > /dev/null
  if [ $? != 0 ] ; then 
    myecho "does not exist. (ERROR)"
    return 1
  else
    myecho "exists. (OK)"
    return 0
  fi 
} 

check_group_exist ()
{
  necho "Group $1 "
  egrep "^${1}:"  /etc/group > /dev/null
  if [ $? != 0 ] ; then 
    myecho "does not exist. (ERROR)"
    return 1
  else
    myecho "exists. (OK)"
    return 0
  fi 
} 

check_user_belongs_to_group ()
{
  necho "User $1 "
  egrep "^${2}:"  /etc/group | grep "$1" > /dev/null
  if [ $? != 0 ] ; then 
    myecho "does not belong to group $2 . (ERROR)"
    return 1
  else
    myecho "belong to group $2. (OK)"
    return 0
  fi 
} 

check_command_exist ()
{
  which $1 > /dev/null 2>&1
  if [ $? != 0 ] ; then 
    myecho "Command $1 does not exist. (ERROR)"
    return 1
  else
    myecho "Command $1 exists. (OK)"
    return 0
  fi 
} 

check_vlt_ccstype ()
{
  if [ -z "$VLTSW_CCSTYPE" ]; then
    echo "Environment VLTSW_CCSTYPE not defined (ERROR)"
    return 1
  fi
  if [ "$1" != "$VLTSW_CCSTYPE" ]; then
    echo "Environment VLTSW_CCSTYPE not defined as expected $1 (ERROR)"
    return 1
  else
    echo "Environment VLTSW_CCSTYPE is defined as expected $1 (OK)"
    return 0
  fi
}
