package org.eso.dfs.services.data;

import java.io.File;
import java.util.List;

/**
 * @author Nick Kornweibel
 *
 * This interface defines a record of data generated by the 
 * AssociationService and is typically related to a single (fits/tfits)
 * frame. 
 * 
 * "Association-technologies" is a work "in progress" at ESO.
 * When the project is more advanced, and there are accepted rules
 * for classification, grouping, etc, the Gasgano AssociationService
 * will be modified to conform to these rules.
 * 
 * The key elements of the AssociationNode are:
 * 
 * Classification: 
 * Classification is a value or set of values determined by the
 * presence and values of keywords in the file.
 * 
 * Array of String objects specifying the classification
 * of the file. This is currently only a single value, in
 * element 0, and Gasgano only supports a single
 * classification value.
 * 
 * If the file can not be classified or has no classification
 * the array will be size 0.
 * 
 * Grouping:
 * Grouping is a value formed by the combination of values of
 * specific keywords in the file. Files with the same grouping
 * value are part of the same "group".
 * 
 * Array of String objects specifying the grouping value of the
 * file. This is currently only a single value. in element 0,
 * and Gasgano only supports a single grouping value.
 * 
 * If the file can not be grouped or has not grouping value the
 * array will be size 0.
 * 
 * Category:
 * The file category represents the basic mode under which the frame was
 * created and is (typically) either "RAW" or "CALIB".
 * 
 * If the file type can not be determined it will be null
 * 
 * Instrument Name:
 * The name of the instrument used to create the frame. If the
 * instrument could not be determined it will be null.
 */
public interface AssociationNode {

    /**
     * Returns the classification.
     * @return String[]
     */
    public String[] getClassification();

    /**
     * Returns the grouping.
     * @return String[]
     */
    public String[] getGrouping();

    /**
     * Returns the instrumentName.
     * @return String
     */
    public String getInstrumentName();

    /**
     * Returns the category.
     * @return String
     */
    public String getCategory();

    /**
     * Sets the classification.
     * @param classification The classification to set
     */
    public void setClassification(String[] classification);

    /**
     * Sets the grouping.
     * @param grouping The grouping to set
     */
    public void setGrouping(String[] grouping);

    /**
     * Sets the instrumentName.
     * @param instrumentName The instrumentName to set
     */
    public void setInstrumentName(String instrumentName);

    /**
     * Sets the category.
     * @param type The category to set
     */
    public void setCategory(String type);
}
