/**
 * @author M. Zamparelli
 * @version 1.0
 * A class for defining a component of an Observation,
 * as taken from a database
 */

package org.eso.gasgano.datamodel.gui;

import java.awt.Color;
import java.awt.Font;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;

import org.eso.gasgano.datamodel.FileDescription;
import org.eso.gasgano.datamodel.database.DBContainer;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.gui.GUIUtils;
import org.eso.gasgano.keyword.ClassificationRule;
import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.KeywordBlock;
import org.eso.gasgano.keyword.KeywordSet;
import org.eso.gasgano.keyword.RuleTable;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.RADECConverter;
import org.eso.gasgano.tools.SortedHashtable;

/**
 * this class has no independence of its own<br>
 * it relies heavily on the DBDataModel, in particular on 
 * any ResultSet of an open transaction
 * @author: Michele Zamparelli
 */

public class DBOBComponent implements KeywordSet {
    
    private static final double threeMil = 3600000.0;
    
    private SortedHashtable hTable = null;
    private Vector keywordArray = null;
    // position in the tree
    private SortedHashtable program = null;
    private SortedHashtable ob = null;
    private String telescope = null;

    private DBContainer[] obs = new DBContainer[3];

    private String fileName = null;
    private static Vector keywordFilter;
    private static FileDescription descFactory = null;
    private String classification = null;
    private boolean isPipeProduct = false;

    private static Keyword[] defaultKeywordQuery = null;
    private static final int numberIdKeys = 3;
    private String[] columnDisplayData = null;

    private Font labelFont = null;
    
    
    private static JLabel rawLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(GUIUtils.rawGif)),
            (int) JLabel.LEFT_ALIGNMENT);


    public DBOBComponent() {
        // draft
        hTable = new SortedHashtable();
        keywordArray = new Vector(0);
    }

    /**
     * this method must be called from the DBDataModel, prior
     * to loading all necessary keywords from the various tables<p>
     * If called before, classification and displayed keywords may not
     * be correct<p>
     */

    public void init() {
        if (defaultKeywordQuery == null) {
            setDefaultKeywordQuery(null);
        }
        fileName = getKeyword(Keyword.ARCFILE).getValue();

        classify();
        addKeyword(
            new Keyword(
                Keyword.CLASSIFICATION_METAKEYWORD,
                getClassification()));
        fillDefaultKeywords();
        setDescription();
        setTelescope(getKeyword(Keyword.TELESCOP).getValue());
    }

    public String toString() {
        return fileName;
    }

    public String getId() {
        return toString();
    }

    /**
     * Get the primary header as a keyword block
     * @return KeywordBlock
     */
    public KeywordBlock getHeaderKeywordBlock() {
        return new KeywordBlock(keywordArray, hTable);
    }

    private void fillDefaultKeywords() {
        for (int z = 0; z < defaultKeywordQuery.length; z++) {
            Keyword tmpKey = getKeyword(defaultKeywordQuery[z].getName());
            defaultKeywordQuery[z].setValue(
                ((tmpKey == null) ? "" : tmpKey.getValue()));
        }
    }

    public void setDescription() {
        if (getDefaultFileDescription() != null) {
            getDefaultFileDescription().setDescription(this);
        }
    }

    public static void setDefaultFileDescription(FileDescription d) {
        descFactory = d;
    }

    /**
     * @return The FileDescription object used to set the derscriptions of
     * all OBComponent objects.
     */

    public static FileDescription getDefaultFileDescription() {
        return descFactory;
    }

    public static Keyword[] getDefaultKeywordQuery() {
        return defaultKeywordQuery;
    }

    /**
     * @param k The array of Keywords to be filled in when reading in a
     * Fits file header. This array may be added to.
     */
    public static void setDefaultKeywordQuery(Keyword[] k) {
        int klen = 0;
        if (k != null) {
            klen = k.length;
        }
        defaultKeywordQuery = new Keyword[numberIdKeys + klen];
        defaultKeywordQuery[0] = new Keyword(Keyword.PROG_ID, null);
        defaultKeywordQuery[1] = new Keyword(Keyword.OBS_ID, null);
        defaultKeywordQuery[2] = new Keyword(Keyword.ARCFILE, null);
        if (k != null) {
            System.arraycopy(k, 0, defaultKeywordQuery, numberIdKeys, k.length);
        }
    }

    /**
     * @param k The array of Keywords to be added to the
     * default keyword query set.
     */
    public static void addToDefaultKeywordQuery(Keyword[] k) {
        if (defaultKeywordQuery == null) {
            setDefaultKeywordQuery(k);
        } else if (k != null) {
            Keyword[] temp = defaultKeywordQuery;
            defaultKeywordQuery = new Keyword[temp.length + k.length];
            System.arraycopy(temp, 0, defaultKeywordQuery, 0, temp.length);
            System.arraycopy(k, 0, defaultKeywordQuery, temp.length, k.length);
        }
    }

    public String[] getDisplayData() {
        return columnDisplayData;
    }

    public void setDisplayData(String[] cols) {
        columnDisplayData = cols;
    }

    //// tree insertion and retrieval methods ////////////////////

    public void setProgram(SortedHashtable p) {
        program = p;
    }

    public void setTelescope(String tel) {
        telescope = tel;
    }

    /**
     * @param o The observation under which this component belongs.
     */
    public void setObservation(DBContainer o, int mode) {
        obs[mode] = o;
    }

    /**
     * @return The observation under which this component belongs.
     */
    public DBContainer getObservation(int mode) {
        return obs[mode];
    }

    public SortedHashtable getProgram() {
        return program;
    }

    public String getTelescope() {
        if (telescope != null && !telescope.equals(new String(""))) {
            return telescope;
        } else {
            return DFSDataModel.UNKNOWN_TELESCOPE_ID;
        }
    }

    public Vector getVector() {
        return keywordArray;
    }

    public void classify() {
        classify(DFSDataModel.getDataModel().getRuleTable());
        if (getDefaultFileDescription() != null) {
            // getDefaultFileDescription().classificationChanged(this);
        }
    }

    /**
     * @param t The RuleTable to be used to classify this file.
     */
    public void classify(RuleTable t) {

        long startTime = 0;
        if (PropertyDB.getInstance().isDebug(PropertyDB.debugTiming)) {
            startTime = System.currentTimeMillis();
        }
        classification = null;
        if (t != null) {
            Enumeration e = null;
            Keyword instrument = getKeyword(Keyword.INSTRUMENT);
            if (instrument != null) {
                // request the Rules matching our instrument.
                e = t.elements(instrument.getValue());
            } else {
                e = t.elements();
            }
            while (e.hasMoreElements()) {
                ClassificationRule rule = (ClassificationRule) e.nextElement();
                // System.out.println(rule.getInstrument());
                // System.out.println(rule.ruleToString());
                if (rule != null && rule.match(this)) {
                    classification = rule.getClassification();
                    break;
                }
            }
        }
        if (classification == null) {
            classification = "UNCLASSIFIED";
        }
    }

    public String getClassification() {
        return classification;
    }

    /**
     * @param kwv The set of Keywords to be displayed in the 
     * graphic component.
     */
    public static void setKeywordFilter(Vector kwv) {
        keywordFilter = kwv;
    }

    /**
     * @return The set of Keywords to be displayed in the 
     * graphic component.
     */
    public static Vector getKeywordFilter() {
        return keywordFilter;
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = null;
        // Use default colors for now.
        Color foreGround = null;
        cell = rawLabel;
        foreGround = GUIUtils.dataBaseColor;
        cell.setText(toString());
        cell.setForeground(foreGround);
        cell.setFont(getLabelFont());
        return cell;
    }

    public Font getLabelFont() {
        if (labelFont == null) {

            String fontString =
                (String) (PropertyDB.getInstance().getProperty("COMPONENT_FONT"));
            String fontName =
                fontString.substring(0, fontString.indexOf(":", 0));
            String fontSize =
                fontString.substring(
                    fontString.indexOf(":", 0) + 1,
                    fontString.length());

            labelFont =
                new Font(fontName, Font.PLAIN, Integer.parseInt(fontSize));
            // new Font("Dialog", Font.PLAIN, 12);
        }
        return labelFont;
    }

    /**
     * @return The JComponent for displaying this OB Component.
     */
    public JComponent getDisplayComponent() {
        JComponent rc = null;
        String title = null;
        title = toString() + " " + getClassification();
        rc = new FitsFileDisplay(title, new FitsTableModel(this));
        return rc;
    }

    /**
     * Implementation of the KeywordSet interface.
     * @param s The name of the Keyword to find.
     * @return The Keyword located, or null if not located.
     */
    public Keyword getKeyword(String name) {
        Keyword rc = null;
        rc = (Keyword) hTable.get(name);
        return rc;
    }

    /**
     * Implementation of the KeywordSet interface.
     * @param k The Keyword to be added.
     */
    public void addKeyword(Keyword k) {
        if (k != null) {

            if (k.getName().equals("RA")) {
                double tmp = Double.valueOf(k.getValue()).doubleValue();
                tmp = tmp / threeMil;
                k.setValue(RADECConverter.RAconvert(String.valueOf(tmp)));
            }
            if (k.getName().equals("DEC")) {
                double tmp = Double.valueOf(k.getValue()).doubleValue();
                tmp = tmp / threeMil;
                k.setValue(RADECConverter.DECconvert(String.valueOf(tmp)));
            }

            try {
                hTable.put(k.getName(), k);
            } catch (NullPointerException ex) {
                System.out.println("error:" + ex.toString());
            }
            keywordArray.addElement(k);
        }
    }

    /**
     * Implementation of the KeywordSet interface.
     * @param k The Keyword to be deleted. This method does nothing.
     */
    public void removeKeyword(String k) {
        return;
    }

    /**
     * Implementation of the KeywordSet interface.
     * @param k The Keyword to be replaced. This method does nothing.
     */
    public void replaceKeyword(Keyword k) {
        return;
    }

    public void freeHeaderMemory() {
        hTable.clear();
        keywordArray.removeAllElements();
    }

}
