/**
 * @author M. Zamparelli
 * @version 1.0
 * A class for defining a TFitsFile Observation component.
 * copies many of the methods of DFSTextFileOBComponent
 */

package org.eso.gasgano.datamodel.gui;

import java.awt.Color;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;

import org.eso.gasgano.datamodel.DataModelChangeListener;
import org.eso.gasgano.datamodel.DataModelChangeNotifier;
import org.eso.gasgano.datamodel.DataModelEvent;
import org.eso.gasgano.datamodel.FrameInfo;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.gui.GUIUtils;
import org.eso.gasgano.properties.PropertyDB;

public class TFitsFileOBComponent
    extends KeywordContainerOBComponent
    implements DataModelChangeListener {

    private DataModelChangeNotifier eventSource = null;
    private boolean isSelfLocalizable = false;

    
    private static JLabel tfitsLabel =
        new JLabel(
            new ImageIcon(TFitsFileOBComponent.class.getResource(GUIUtils.tfitsGif)),
            (int) JLabel.LEFT_ALIGNMENT);

    /**
     * @param address The path of the fits file.
     */
    public TFitsFileOBComponent(String address) {
        super(address);
        long startTime = 0;
        if (PropertyDB.getInstance().isDebug(PropertyDB.debugTiming)) {
            startTime = System.currentTimeMillis();
        }
        if (getStatus() == DFSFile.FILE_VALID) {
            // add this OBComponent to the DataModel.
            addToDataModel(
                defaultKeywordQuery[0].getValue(),
                defaultKeywordQuery[1].getValue(),
                myFile.getTelescope());

            if (PropertyDB.getInstance().isDebug(PropertyDB.debugTiming)) {
                initTime += (System.currentTimeMillis() - startTime);
                initNumber++;
            }
        }
    }

    /**
     * @param progId The program id under which this is t be added in the
     * DataModel.
     * @param obId The observation id under which this is to be added in
     * the DataModel.
     * Adding a FitsFileOBComponent into a DataModel - the program and
     * observation should be determined from the Keywords - otherwise
     * they are set to Unknown values.
     */
    public void addToDataModel(
        String progId,
        String obId,
        String telescopeId) {
        DFSDataModel dm = DFSDataModel.getDataModel();
        if (!dm.passFilter(this)) {
            myFile.setStatus(DFSFile.FILE_FILTERED);
            return;
        }
        if (progId == null || obId == null) {
            insertInDataModel();
        } else {
            String dir = myFile.getPath().getParent();
            dm.addObComponent(progId, obId, dir, telescopeId, this);
            isSelfLocalizable = true;
        }

        // ascii files are the result of pipe line reductions.
        setPipeProduct(true);
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = null;
        // Use default colors for now.
        Color foreGround = null;
        cell = tfitsLabel;
        foreGround = Color.red;
        cell.setText(toString());
        cell.setForeground(foreGround);
        cell.setFont(getLabelFont());
        return cell;
    }

    public String getArcfile() {
        return new String("");
    }

    public void classify() {
        myFile.classify(DFSDataModel.getDataModel().getRuleTable());
        if (getDefaultFileDescription() != null) {
            getDefaultFileDescription().classificationChanged(this);
        }
    }

    /**
     * @param e The event to be processed.
     * Either the Component we used to get prog/ob info has
     * changed or we had no such Component and were in the Unknown group, 
     * and new files have been added to the DataModel,
     * either way, we need to remove
     * this from the data model and re-add this.
     */
    public void dataModelChanged(DataModelEvent e) {
        if (e.getEvent() == DataModelEvent.FILE_DELETED
            || e.getEvent() == DataModelEvent.FILE_ADDED
            || e.getEvent() == DataModelEvent.FILE_UPDATED) {
            if (PropertyDB.getInstance().isDebug(PropertyDB.debugEvent)) {
                System.out.println(getId() + " received dataModelChangeEvent.");
            }
            if (!isSelfLocalizable && e.getFile() != this) {
                insertInDataModel();
            }
        }
    }

    /**
     * Adding a DFSTextFileOBComponent into a DataModel:
     * Search for an entry in the data model which has the
     * same base name as this file. If located, add this into the
     * DataModel using the prog and ob info for the located
     * component, and add this as a DataModelChange listener to 
     * that Component, so that the procedure is repeated if the 
     * data source changes.
     * If no file is located, add under "Unknown" catagories and add this
     * as a DataModelChangeListener to the DataModel, 
     * to repeat the procedure if new files are added.
     * If the component is current a part of the data model, and it is
     * determined that it must be moved, it will be removed, and then added.
     */
    private void insertInDataModel() {
        DFSDataModel dm = DFSDataModel.getDataModel();
        String progId = null;
        String obId = null;
        String telescopeId = null;
        if (!dm.passFilter(this)) {
            myFile.setStatus(DFSFile.FILE_FILTERED);
            return;
        }
        OBComponent dataSource = getIdentifyingComponent();
        DataModelChangeNotifier notifier = null;
        if (dataSource == null) {
            progId = DFSDataModel.UNKNOWN_PROGRAM_ID;
            obId = DFSDataModel.UNKNOWN_OBSERVATION_ID;
            telescopeId = DFSDataModel.UNKNOWN_TELESCOPE_ID;
            notifier = dm;
        } else {
            FrameInfo frameInfo = new FrameInfo();
            dm.provideFrameInfo((KeywordContainerOBComponent) dataSource, frameInfo);
            telescopeId = frameInfo.telescopeId;
            progId = frameInfo.programId;
            obId = frameInfo.obsId;
            notifier = dataSource;
        }
        if (storeProgram != null
            && storeProgram.equals(progId)
            && storeObservation != null
            && storeObservation.equals(obId)
            && storeTelescope != null
            && storeTelescope.equals(telescopeId)) {
            // we have not changed prog/ob/date...
        } else {
            // remove this from the model if in the model
            // and add this, if valid.
            if (eventSource != null) {
                removeFromDataModel();
            }
            if (getStatus() == DFSFile.FILE_VALID) {
                String dir = myFile.getPath().getParent();
                dm.addObComponent(progId, obId, dir, telescopeId, this);
                storeProgram = (storeProgram == null) ? progId : storeProgram;
                storeObservation =
                    (storeObservation == null) ? obId : storeObservation;
                storeTelescope =
                    (storeTelescope == null) ? telescopeId : storeTelescope;
                registerToNotifier(notifier);
            }
        }
    }

    /**
     * @return The OBComponent (A FitsFileOBComponent) which can be
     * used to get prog id, ob id info.
     * Or null if not appropriate component is located.
     */
    private OBComponent getIdentifyingComponent() {
        OBComponent rc = null;
        DFSDataModel dm = DFSDataModel.getDataModel();
        Vector dataSourceSet =
            dm.getObComponentByBaseName(myFile.getBaseName());
        if (dataSourceSet != null && dataSourceSet.isEmpty() == false) {
            Enumeration e = dataSourceSet.elements();
            while (rc == null && e.hasMoreElements()) {
                rc = (OBComponent) e.nextElement();
                if (rc instanceof FitsFileOBComponent == false) {
                    // we only want to find a FitsFile.
                    rc = null;
                }
            }
        }
        return rc;
    }

    /**
     * @param n The DataModelChangeNotifier to which this is to be
     * registered. Register this as a listener to the DataChangeNotifier 
     * removing the old registration if required. This method assumes there 
     * is only one DataChangeNotifier regisitered to at one time.
     */
    private void registerToNotifier(DataModelChangeNotifier n) {
        if (eventSource != n) {
            if (eventSource != null) {
                eventSource.removeDataModelChangeListener(this);
            }
            if (n != null) {
                n.addDataModelChangeListener(this);
            }
            eventSource = n;
        }
    }

    public void cleanup() {
        super.cleanup();
        // remove our notification registration.
        registerToNotifier(null);
    }

} // end of class TFitsFileOBComponent.java

// end of file TFitsFileOBComponent.java
