/**
 * @author Nick Kornweibel.
 * 
 * Java class to enable display and editing of classification rules.
 * The RuleTable must be read in and presented to the class.
 */

package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.dfs.gui.SwingFileChooser;
import org.eso.gasgano.keyword.ClassificationRule;
import org.eso.gasgano.keyword.EqKeywordExpr;
import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.KeywordExpression;
import org.eso.gasgano.keyword.RuleTable;

public class ClassificationBuilder
    extends JFrame
    implements ActionListener, ListSelectionListener {

    private static final String importLabel = "Append other rule file";
    private static final String quitLabel = "Dismiss";
    private ExpressionBuilder exprBuilder;
    private JScrollPane tableView;
    private JTable table;
    private JSplitPane mainPane;
    private JPanel lowerPanel;
    private JPanel rulePanel;
    private JFileChooser fileChooser;
    private boolean exitOnDismiss = false;

    private ClassificationRule selectedRule;

    private JButton addRule;
    private JButton deleteRule;
    private JButton saveRule;
    private JButton cancelRule;
    private JButton dismissPanel;
    private JButton moveUp;
    private JButton moveDown;

    private JTextField instrumentEntry;
    private JTextField classificationEntry;

    private RuleTable ruleTable;
    private int previousSelectionIndex = -1;
    private boolean selectionCancelled = false;

    // the colour of the rule panel is taken from the colour of the expression
    // builder panel.
    private static Color rulePanelColor =
        ExpressionBuilder.getDefaultPanelColor();

    /**
     * Main method - used to run the classification rule builder in
     * stand alone mode. the command argument expected is the path to the
     * classification rule file (serialized RuleTable).
     * @param args
     */
    public static void main(String args[]) {
        String ruleTablePath = null;
        if (args.length < 1) {
            System.out.println("usage: <path to rule table>");
            System.exit(0);
        } else {
            ruleTablePath = args[0];
        }
        RuleTable rt = RuleTable.readFile(ruleTablePath);
        if (rt == null) {
            System.out.println(
                "Failed to read/locate: "
                    + ruleTablePath
                    + " starting with an empty table.");
            rt = new RuleTable(ruleTablePath);
        }

        ClassificationBuilder s = new ClassificationBuilder(rt);
        s.setExitOnDismiss(true);

        s.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        s.pack();
        s.setSize(780, 640);
        // required by my window manager at least...
        s.setLocation(100, 300);
        s.setVisible(true);
    }

    public ClassificationBuilder(RuleTable rt) {
        super("Classification Rule Builder: " + rt.getTablePath());
        ruleTable = rt;
        init();
    }

    private void init() {
        mainPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

        rulePanel = new JPanel(new BorderLayout());
        rulePanel.setBackground(rulePanelColor);
        rulePanel.setBorder(new LineBorder(Color.DARK_GRAY, 2));
        lowerPanel = new JPanel(new BorderLayout());
        lowerPanel.setBackground(rulePanelColor);

        table = new JTable(ruleTable);
        table.getColumnModel().getColumn(0).setMinWidth(7);
        table.getColumnModel().getColumn(0).setPreferredWidth(7);
        table.getColumnModel().getColumn(1).setMinWidth(7);
        table.getColumnModel().getColumn(1).setPreferredWidth(7);
        table.getColumnModel().getColumn(2).setMinWidth(350);
        table.getColumnModel().getColumn(2).setPreferredWidth(350);

        ListSelectionModel lsm = table.getSelectionModel();
        if (lsm != null) {
            lsm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            lsm.addListSelectionListener(this);
        }

        tableView = new JScrollPane(table);
        tableView.setPreferredSize(new Dimension(600, 500));
        tableView.setMinimumSize(new Dimension(100, 100));
        mainPane.setTopComponent(tableView);

        JPanel p = new JPanel(new GridLayout(2, 2));
        p.setBackground(rulePanelColor);
        p.add(new JLabel("Instrument: ", JTextField.RIGHT));
        instrumentEntry = new JTextField(20);
        p.add(instrumentEntry);
        p.add(new JLabel("Classification: ", JTextField.RIGHT));
        classificationEntry = new JTextField(20);
        p.add(classificationEntry);
        rulePanel.add(p, BorderLayout.NORTH);

        exprBuilder = new ExpressionBuilder(this);
        rulePanel.add(exprBuilder, BorderLayout.CENTER);

        p = new JPanel();
        addRule = new JButton("Add Rule to table");
        addRule.addActionListener(this);
        deleteRule = new JButton("Delete Selected Rule");
        deleteRule.addActionListener(this);
        saveRule = new JButton("Replace Selected Rule");
        saveRule.addActionListener(this);
        cancelRule = new JButton("Reset Displayed Rule");
        cancelRule.addActionListener(this);

        moveUp = new JButton("Move Up");
        moveUp.addActionListener(this);

        moveDown = new JButton("Move Down");
        moveDown.addActionListener(this);

        p.add(moveUp);
        p.add(moveDown);
        p.add(addRule);
        p.add(deleteRule);
        p.add(saveRule);
        p.add(cancelRule);

        lowerPanel.add(p, BorderLayout.SOUTH);

        lowerPanel.add(rulePanel, BorderLayout.CENTER);
        mainPane.setBottomComponent(lowerPanel);
        getContentPane().add(mainPane, BorderLayout.CENTER);
        addMenu();
        selectedRule = null;
        displayRule(selectedRule);
    }

    public void setRuleTable(RuleTable rt) {
        setTitle("Classification Rule Builder: " + rt.getTablePath());
        ruleTable = rt;
        init();
        pack();
    }

    private void addMenu() {
        MenuBar mb = new MenuBar();
        Menu menu = new Menu("File");

        MenuItem mi = null;

        mi = new MenuItem(importLabel);
        mi.addActionListener(this);
        menu.add(mi);

        mi = new MenuItem(quitLabel);
        mi.addActionListener(this);
        menu.add(mi);

        mb.add(menu);
        setMenuBar(mb);
    }

    /**
     * Call back for processing table (rule) selection.
     * @see javax.swing.event.ListSelectionListener#valueChanged(ListSelectionEvent)
     */
    public void valueChanged(ListSelectionEvent e) {
        int i = -1;
        if (e != null && e.getValueIsAdjusting() == false) {
            // first we check whether it is a internal selection
            if (selectionCancelled) {
                selectionCancelled = false;
                return;
            }

            ListSelectionModel lsm = (ListSelectionModel) e.getSource();
            if (lsm.isSelectionEmpty()) {
                //no rows are selected
                selectedRule = null;
                displayRule(selectedRule);
            } else {
                if (!cancelChanges()) {
                    selectionCancelled = true;
                    lsm.setSelectionInterval(
                        previousSelectionIndex,
                        previousSelectionIndex);
                    return;
                }
                i = lsm.getMinSelectionIndex();
                previousSelectionIndex = i;
                selectedRule = (ClassificationRule) ruleTable.elementAt(i);

                displayRule(selectedRule);
            }

        }
    }

    /**
     * Determine if there have been changes to the current classification
     * rule, and if, prompt the user before continuing. Continuing will
     * typically delete the changes done.
     * @return boolean true if changes are to be cancelled.
     */
    private boolean cancelChanges() {
        boolean rc = true;
        if (selectedRule != null
            && exprBuilder.getKeywordExpression() != null) {
            ClassificationRule cr = new ClassificationRule();
            cr.setClassification(classificationEntry.getText());
            cr.setInstrument(instrumentEntry.getText());
            cr.setRule(exprBuilder.getKeywordExpression());

            if (!selectedRule.equals(cr)) {
                String tmpString =
                    "There are unsaved changes in the selected rule."
                        + "\nOK to discard changes to "
                        + cr.getInstrument()
                        + " rule \""
                        + cr.getClassification()
                        + "\" ?";
                rc =
                    (JOptionPane
                        .showConfirmDialog(
                            this,
                            tmpString,
                            "Warning",
                            JOptionPane.OK_CANCEL_OPTION)
                        == JOptionPane.OK_OPTION);
            }
        }
        return rc;
    }

    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        ListSelectionModel lsm = table.getSelectionModel();

        if (a.getSource() == saveRule) {
            if (selectedRule == null) {
                userMessage("No rule selected to replace");
                return;
            }
            String classif = classificationEntry.getText();
            if (classif == null || classif.length() < 1) {
                userMessage("Classification field is empty");
                return;
            }
            String inst = instrumentEntry.getText();
            if (inst == null || inst.length() < 1) {
                userMessage("Instrument field is empty");
                return;
            }
            KeywordExpression expr = exprBuilder.getKeywordExpression();
            if (expr != null) {
                selectedRule.setClassification(classif);
                selectedRule.setInstrument(inst);
                selectedRule.setRule(expr);
                saveFile();
                //selectedRule = null;
                //table.clearSelection();
                //displayRule(selectedRule);
                ruleTable.fireTableDataChanged();
            } else {
                userMessage("Expression not saved.");
            }
        } else if (a.getSource() == addRule) {
            String classif = classificationEntry.getText();
            if (classif == null || classif.length() < 1) {
                userMessage("Classification field is empty");
                return;
            }
            String inst = instrumentEntry.getText();
            if (inst == null || inst.length() < 1) {
                userMessage("Instrument field is empty");
                return;
            }
            KeywordExpression expr = exprBuilder.getKeywordExpression();
            if (expr != null) {
                ClassificationRule c = new ClassificationRule();
                c.setClassification(classif);
                c.setInstrument(inst);
                c.setRule(expr);
                ruleTable.addElement(c);
                saveFile();

                //selectedRule = null;
                table.clearSelection();
                int row = ruleTable.getRowCount() - 1;
                if (row >= 0) {
                    lsm.setSelectionInterval(row, row);
                } else {
                    lsm.clearSelection();
                }
                displayRule(c);
            } else {
                userMessage("Expression not saved.");
            }
        } else if (a.getSource() == deleteRule) {
            if (selectedRule != null) {
                int index = lsm.getMinSelectionIndex();
                ruleTable.removeElement(selectedRule);
                saveFile();
                if (index < ruleTable.getRowCount()) {
                    table.clearSelection();
                    if (index >= 0) {
                        lsm.setSelectionInterval(index, index);
                    } else {
                        lsm.clearSelection();
                    }
                } else {
                    table.clearSelection();
                    if (index > 0) {
                        lsm.setSelectionInterval(index - 1, index - 1);
                    } else {
                        lsm.clearSelection();
                    }
                }
            } else {
                userMessage("No rules selected to delete");
            }
        } else if (a.getSource() == cancelRule) {
            //selectedRule = null;
            //table.clearSelection();
            displayRule(selectedRule);
        } else if (
            a.getActionCommand() == quitLabel
                || a.getSource() == dismissPanel) {
            setVisible(false);
            if (isExitOnDismiss()) {
                System.exit(0);
            }
        } else if (a.getSource() == moveUp) {
            if (selectedRule == null) {
                userMessage("No rule selected to move");
                return;
            }
            if (!cancelChanges()) {
                return;
            }
            if (ruleTable.moveElementUp(selectedRule)) {
                saveFile();
                // Java 1.1.8 int index = lsm.getMinSelectionIndex();
                int index = previousSelectionIndex;
                if (index > 0) {
                    lsm.setSelectionInterval(index - 1, index - 1);
                    table.scrollRectToVisible(
                        table.getCellRect(index - 1, 0, true));
                } else {
                    lsm.clearSelection();
                }
            }
        } else if (a.getSource() == moveDown) {
            if (selectedRule == null) {
                userMessage("No rule selected to move");
                return;
            }
            if (!cancelChanges()) {
                return;
            }
            if (ruleTable.moveElementDown(selectedRule)) {
                saveFile();
                // Java 1.1.8 int index = lsm.getMinSelectionIndex();
                int index = previousSelectionIndex;
                lsm.setSelectionInterval(index + 1, index + 1);
                table.scrollRectToVisible(
                    table.getCellRect(index + 1, 0, true));
            }
        } else if (a.getActionCommand() == importLabel) {
            importFile();
        }
    }

    private void displayRule(ClassificationRule c) {
        if (c == null || c.getRule().getExpressionTree() == null) {
            classificationEntry.setText("");
            instrumentEntry.setText("");
            KeywordExpression k =
                new KeywordExpression(
                    new EqKeywordExpr(
                        new Keyword("KEYWORD", null),
                        new String("value")));
            exprBuilder.setKeywordExpression(k);
        } else {
            classificationEntry.setText(c.getClassification());
            instrumentEntry.setText(c.getInstrument());
            exprBuilder.setKeywordExpression(c.getRule());
        }
        deleteRule.setEnabled(c != null);
        moveUp.setEnabled(c != null);
        moveDown.setEnabled(c != null);
        saveRule.setEnabled(selectedRule != null);
    }

    public void userMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg);
    }

    private void importFile() {
        String startPath = null;
        try {
            startPath = new File(ruleTable.getTablePath()).getParent();
        } catch (Exception e) {
            System.out.println(e);
            System.out.println("rules file in preferences not correct");
        }
        if (startPath == null) {
            startPath = System.getProperties().getProperty("user.home");
        }
        if (fileChooser == null) {
            fileChooser = new SwingFileChooser(startPath);
            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            fileChooser.setApproveButtonText("Append");
            fileChooser.setMultiSelectionEnabled(false);

            fileChooser.setDialogTitle("Select rule file to be appended");
        }
        // else { 
        //    fileChooser.setCurrentDirectory(new File(startPath));
        //}

        if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            File selectedFile = null;
            selectedFile = fileChooser.getSelectedFile();
            ruleTable.importFile(selectedFile.toString());
            saveFile();
        }
    }

    private void saveFile() {
        try {
            ruleTable.writeFile();
        } catch (IOException e) {
            userMessage("Error saving file: " + ruleTable.getTablePath());
        }
    }
    /**
     * Returns the exitOnDismiss. If true, the dismiss button will
     * exit the JVM. This should be set to true if the editor is being
     * used in stand-alone mode.
     * @return boolean
     */
    public boolean isExitOnDismiss() {
        return exitOnDismiss;
    }

    /**
     * Sets the exitOnDismiss.
     * @param exitOnDismiss The exitOnDismiss to set
     */
    public void setExitOnDismiss(boolean exitOnDismiss) {
        this.exitOnDismiss = exitOnDismiss;
    }

}

// end of file ClassificationBuilder.java
