/**
 * @author Nick Kornweibel
 * 
 * A Scrolling Jtree JPanel widget for building KeywordExpressions
 */

package org.eso.gasgano.gui;

import gnu.regexp.RE;
import gnu.regexp.REException;
import gnu.regexp.REMatch;
import gnu.regexp.RESyntax;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.eso.dfs.gui.RecallingComboBox;
import org.eso.dfs.gui.SharedListComboBoxModel;
import org.eso.gasgano.keyword.AndKeywordExpr;
import org.eso.gasgano.keyword.BinaryKeywordExpr;
import org.eso.gasgano.keyword.DefaultBoolKeywordExpr;
import org.eso.gasgano.keyword.EqKeywordExpr;
import org.eso.gasgano.keyword.GEKeywordExpr;
import org.eso.gasgano.keyword.GTKeywordExpr;
import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.KeywordExpression;
import org.eso.gasgano.keyword.LEKeywordExpr;
import org.eso.gasgano.keyword.LTKeywordExpr;
import org.eso.gasgano.keyword.LikeKeywordExpr;
import org.eso.gasgano.keyword.NEqKeywordExpr;
import org.eso.gasgano.keyword.NotLikeKeywordExpr;
import org.eso.gasgano.keyword.OrKeywordExpr;
import org.eso.gasgano.keyword.SharedKeywordList;

public class ExpressionBuilder extends JPanel {

    private RESyntax theSyntax = new RESyntax(RESyntax.RE_SYNTAX_PERL5);
    private String stringReg = "^\\\"[^\\\"]*\\\"$";
    private RE rhsExp;

    private Frame owner;
    private DefaultMutableTreeNode root;
    private DefaultMutableTreeNode selectedNode;
    private JScrollPane treeView;
    private JTree tree;
    private JPanel rulePanel;

    private Action add;
    private Action delete;
    private Action edit;
    private Action changeToNode;

    private final String addLeafStr = "Add Leaf";
    private final String insertNodeStr = "Insert Node";
    private final String deleteExprStr = "Delete";
    private final String editStr = "Edit";

    static public final String defaultLeafValue = "KEYWORD = \"value\"";
    static public final String defaultNodeValue = AndKeywordExpr.opString;
    
    public static Color panelColor = new Color(180, 180, 180);

    public ExpressionBuilder(Frame o) {
        super(new BorderLayout());
        owner = o;
        init();
    }

    private void init() {
        JPanel p = new JPanel();
        setBackground(panelColor);
        p.setBackground(panelColor);
         
        add = new AbstractAction(addLeafStr) {
            public void actionPerformed(ActionEvent e) {
                treeOpAdd();
            }};

        changeToNode = new AbstractAction(insertNodeStr) {
            public void actionPerformed(ActionEvent e) {
                treeOpChangeToNode();
            }};

        delete = new AbstractAction(deleteExprStr) {
            public void actionPerformed(ActionEvent e) {
                treeOpDelete();
            }};

        edit = new AbstractAction(editStr) {
            public void actionPerformed(ActionEvent e) {
                treeOpEdit();
            }};

        JButton b;
        b = new JButton(add);
        b.setBackground(panelColor);
        p.add(b);
        
        b = new JButton(changeToNode);
        b.setBackground(panelColor);       
        p.add(b);
 
        b = new JButton(delete);
        b.setBackground(panelColor);       
        p.add(b);

        b = new JButton(edit);
        b.setBackground(panelColor);          
        p.add(b);
        
        add(p, BorderLayout.SOUTH);

        UIDefaults defaults = UIManager.getDefaults();
        defaults.put("Tree.leafIcon", new ImageIcon(""));
        defaults.put("Tree.openIcon", new ImageIcon(""));
        defaults.put("Tree.closedIcon", new ImageIcon(""));
        defaults.put("Tree.hash", Color.black);
        root = new DefaultMutableTreeNode(defaultLeafValue);
        refreshTree();

        try {
            rhsExp = new RE(stringReg);
        } catch (REException ree) {
            System.out.println(
                "ExpressionBuilder: could not instantiate gnu regular expression");
        }
        nodeSelected(null);
    }

    private TreePath getPathToRoot(DefaultMutableTreeNode node) {
        int size = 0;
        TreePath rc = null;
        Vector tmpVect = new Vector();
        Object[] tmpArray = null;

        do {
            tmpVect.addElement(node);
            node = (DefaultMutableTreeNode) (node.getParent());
        } while (node != null);
        size = tmpVect.size();
        tmpArray = new Object[size];
        for (int i = 0; i < size; i++) {
            tmpArray[i] = tmpVect.elementAt(size - i - 1);
        }
        rc = new TreePath(tmpArray);
        return rc;
    }
    
    private void nodeSelected(DefaultMutableTreeNode node) {
        selectedNode = node;
        if (selectedNode != null) {
            edit.setEnabled(true);
            delete.setEnabled(true);
            if (selectedNode.isLeaf()) {
                add.setEnabled(false);
                changeToNode.setEnabled(true);
            } else {
                add.setEnabled(true);
                changeToNode.setEnabled(false);
            }
        } else {
            add.setEnabled(false);
            delete.setEnabled(false);
            changeToNode.setEnabled(false);
            edit.setEnabled(false);
        }
        edit.setEnabled(selectedNode != null);
    }

    public DefaultBoolKeywordExpr convertNodeToKeywordExpr(DefaultMutableTreeNode node) {
        DefaultBoolKeywordExpr rc = null;
        String nodeString = node.getUserObject().toString();
        if (node.getChildCount() >= 2) {
            if (nodeString.equals(AndKeywordExpr.opString)) {
                rc = new AndKeywordExpr();
            } else if (nodeString.equals(OrKeywordExpr.opString)) {
                rc = new OrKeywordExpr();
            } else {
                System.out.println(
                    "convertNodeToKeywordExpr <"
                        + node
                        + "> returning null as it was not AND or OR");
                userMessage(
                    "Error, operator: " + nodeString + " not recognized.");
                rc = null;
            }
        } else {
            System.out.println(
                "convertNodeToKeywordExpr <"
                    + node
                    + "> returning null it has less than 2 children.");
            userMessage(
                "Error, operator: "
                    + nodeString
                    + " must have two or more branches/sub-expressions.");
        }
        return rc;
    }

    public BinaryKeywordExpr convertLeafToKeywordExpr(DefaultMutableTreeNode node) {
        BinaryKeywordExpr rc = null;
        // must be a <val> <op> <val> String - parse it
        // and create the correct expression.
        String exp = node.getUserObject().toString().trim();
        String lhVal = null;
        String rhVal = null;
        String opVal = null;
        int endToken = -1;
        try {
            // extract the left hand value.
            if (exp.indexOf('\"') == 0 && exp.length() > 1) {
                // the lh value is a String (starts with '"').
                endToken = exp.indexOf('\"', 1);
            } else {
                endToken = exp.indexOf(' ') - 1;
            }
            lhVal = exp.substring(0, endToken + 1);
            lhVal = lhVal.trim();

            // remove the lh value and trim the string.
            exp = exp.substring(endToken + 1, exp.length()).trim();

            // extract the operator.
            endToken = exp.indexOf(' ');
            opVal = exp.substring(0, endToken).trim();

            // take the rest as the rh value.
            rhVal = exp.substring(endToken + 1, exp.length()).trim();
        } catch (StringIndexOutOfBoundsException e) {
            opVal = null;
            lhVal = null;
            rhVal = null;
        }
        if (lhVal != null && rhVal != null && opVal != null) {
            Object lhs = KeywordExpression.stringToValue(lhVal);
            Object rhs = KeywordExpression.stringToValue(rhVal);
            if (lhs != null && rhs != null && opVal != null) {
                if (opVal.equals(EqKeywordExpr.opString)) {
                    rc = new EqKeywordExpr(lhs, rhs);
                } else if (opVal.equals(NEqKeywordExpr.opString)) {
                    rc = new NEqKeywordExpr(lhs, rhs);
                } else if (opVal.equals(LTKeywordExpr.opString)) {
                    rc = new LTKeywordExpr(lhs, rhs);
                } else if (opVal.equals(GTKeywordExpr.opString)) {
                    rc = new GTKeywordExpr(lhs, rhs);
                } else if (opVal.equals(LEKeywordExpr.opString)) {
                    rc = new LEKeywordExpr(lhs, rhs);
                } else if (opVal.equals(GEKeywordExpr.opString)) {
                    rc = new GEKeywordExpr(lhs, rhs);
                } else if (opVal.equals(LikeKeywordExpr.opString)) {
                    rc = new LikeKeywordExpr(lhs, rhs);
                } else if (opVal.equals(NotLikeKeywordExpr.opString)) {
                    rc = new NotLikeKeywordExpr(lhs, rhs);
                } else {
                    System.out.println(
                        "convertLeafToKeywordExpr: "
                            + "returning null as operator not found.");
                    userMessage(
                        "Error, operator: " + opVal + " not recognized.");
                }
            }
        } else {
            System.out.println(
                "convertLeafToKeywordExpr: "
                    + "returning null as string not parsed successfully");
            userMessage(
                "Error, expression: "
                    + node.getUserObject().toString()
                    + " could not be parsed.");
        }
        return rc;
    }

    public DefaultBoolKeywordExpr convertToKeywordExpr(DefaultMutableTreeNode node) {
        DefaultBoolKeywordExpr rc = null;
        if (node.getChildCount() > 0) {
            rc = convertNodeToKeywordExpr(node);
            if (rc != null) {
                for (int i = 0; i < node.getChildCount(); i++) {
                    DefaultMutableTreeNode child =
                        convertToKeywordExpr(
                            (DefaultMutableTreeNode) node.getChildAt(i));
                    if (child != null) {
                        rc.add(child);
                    } else {
                        System.out.println(
                            "convertToKeywordExpr:"
                                + " return null as child "
                                + i
                                + " was null");
                        rc = null;
                        break;
                    }
                }
            }
        } else {
            rc = convertLeafToKeywordExpr(node);
        }
        return rc;
    }

    public void setKeywordExpression(KeywordExpression kwe) {
        root = treeCopy(kwe.getExpressionTree());
        refreshTree();
    }

    public KeywordExpression getKeywordExpression() {
        DefaultBoolKeywordExpr exp = convertToKeywordExpr(root);
        KeywordExpression rc = null;
        if (exp != null) {
            rc = new KeywordExpression(exp);
        }
        return rc;
    }

    public static DefaultMutableTreeNode treeCopy(DefaultMutableTreeNode t) {
        DefaultMutableTreeNode rc = new DefaultMutableTreeNode(t.toString());
        Enumeration e = t.children();
        while (e.hasMoreElements()) {
            rc.add(treeCopy((DefaultMutableTreeNode) e.nextElement()));
        }
        return rc;
    }

    void userMessage(String msg) {
        JOptionPane.showMessageDialog(owner, msg);
    }

    /**
     * TODO: Why does the tree have to be removed and re-created simply because
     * the model has been replaced - this should not be required and needs to
     * be investigated.
     * 
     * Refresh the tree widget to reflect a (possibly) new tree model
     */
    private void refreshTree() {
        if (treeView != null) {
            remove(treeView);
        }
        tree = new JTree(root);
        tree.getSelectionModel().setSelectionMode(
            TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.putClientProperty("JTree.lineStyle", "Angled");
        tree.setEditable(false);
        tree.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                int selRow = tree.getRowForLocation(e.getX(), e.getY());
                TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
                if (selPath != null && e.getClickCount() == 2) {
                    treeOpEdit();
                }
            }
        });
        tree.setVisibleRowCount(1000);
        // expand branch.
        for (int i = 0; i < tree.getRowCount(); i++) {
            tree.expandRow(i);
        }
        tree.setExpandsSelectedPaths(false);
        // Register a TreeSelectionListener....
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                    nodeSelected((DefaultMutableTreeNode) (e.getPath().getLastPathComponent()));
                }
            });

        //Create the scroll pane and add the tree to it.
        treeView = new JScrollPane(tree);
        add(treeView, BorderLayout.CENTER);
        validate();
        nodeSelected(null);
    }

    public void treeOpEdit() {
        if (selectedNode == null) {
            return;
        }
        if (selectedNode.getChildCount() == 0) {
            BinaryOpExprEditor editor =
                new BinaryOpExprEditor(owner, selectedNode);
            editor.setModal(true);
            editor.setSize(600, 240);
            Point p = owner.getLocation();
            Dimension d = owner.getSize();
            p.move(p.x + d.width / 2 - 250, p.y + d.height / 2);
            editor.setLocation(p);
            editor.show();
        } else {
            tree.expandPath(tree.getLeadSelectionPath());
            BoolExprEditor editor = new BoolExprEditor(owner, selectedNode);
            editor.setModal(true);
            editor.setSize(300, 120);
            Point p = owner.getLocation();
            Dimension d = owner.getSize();
            p.move(p.x + d.width / 2 - 250, p.y + d.height / 2);
            editor.setLocation(p);
            editor.show();
        }
        refreshTree();
    }

    private void treeOpAdd() {
        if (selectedNode == null) {
            return;
        }
        if (selectedNode.isLeaf()) {
            return;
        }
        DefaultMutableTreeNode child =
            new DefaultMutableTreeNode(defaultLeafValue);
        selectedNode.add(child);
        refreshTree();
    }

    /**
     * Perform a delete on the tree.
     */
    private void treeOpDelete() {
        DefaultMutableTreeNode dmtn = null;
        DefaultMutableTreeNode sibling = null;
        int nodeIndex = 0;
        if (selectedNode == null) {
            return;
        }
        if (selectedNode == root) {
            treeOpChangeToLeaf();
            return;
        }

        if (selectedNode.getNextSibling() == null
            && selectedNode.getPreviousSibling() == null) {
            sibling = (DefaultMutableTreeNode) selectedNode.getParent();
        } else if (selectedNode.getNextSibling() != null) {
            sibling = selectedNode.getNextSibling();
        } else if (selectedNode.getPreviousSibling() != null) {
            sibling = selectedNode.getPreviousSibling();
        } else {
            //System.out.println("Can't find any relatives");
        }

        selectedNode.removeFromParent();
        refreshTree();
        tree.setSelectionPath(getPathToRoot(sibling));
    }

    /**
     * This method is called for delete, if performed on the root node.
     * Delete does not just remove the root node and all subtrees, it moves
     * the first leaf up to this node.
     */
    private void treeOpChangeToLeaf() {
        if (selectedNode == null) {
            return;
        }
        if (selectedNode.isLeaf()) {
            return;
        }
        String previousContent =
            (String) selectedNode.getFirstLeaf().getUserObject();
        selectedNode.removeAllChildren();
        selectedNode.setUserObject(previousContent);
        refreshTree();
    }

    /**
     * insert a node (and second leaf) above the selected leaf. 
     */
    private void treeOpChangeToNode() {
        if (selectedNode == null) {
            return;
        }
        if (selectedNode.isLeaf()) {
            String previousContent = (String) selectedNode.getUserObject();
            selectedNode.setUserObject(defaultNodeValue);
            DefaultMutableTreeNode child =
                new DefaultMutableTreeNode(previousContent);
            selectedNode.add(child);
            child = new DefaultMutableTreeNode(defaultLeafValue);
            selectedNode.add(child);
            refreshTree();
        }
    }

    private boolean isLegalRHS(String rhs) {
        REMatch theMatch = null;
        if (rhs.equals("")) {
            return false;
        }
        if (rhs.indexOf('"') == -1) {
            return true;
        } else {
            theMatch = rhsExp.getMatch(rhs);
            if (theMatch != null) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * Modal dialog for editing a BinaryKeywordExpression.
     */
    class BinaryOpExprEditor extends JDialog implements ActionListener {

        private DefaultMutableTreeNode targetNode;
        private JButton okButton;
        private JButton cancelButton;
        private final String[] operators =
            {
                EqKeywordExpr.opString,
                NEqKeywordExpr.opString,
                LTKeywordExpr.opString,
                LEKeywordExpr.opString,
                GTKeywordExpr.opString,
                GEKeywordExpr.opString,
                LikeKeywordExpr.opString,
                NotLikeKeywordExpr.opString };

        private final String keywordSyntax =
            "Keyword values (upper case with 'dot' notation): "
                + "TPL.ID, ORIGFILE, OBS.NAME, RA";
        private final String stringSyntax =
            "String values (contained within double quotes): "
                + "\"flat_001\", \"SCIENCE\", \"23/04/97\", \"12.3\"";
        private final String numberSyntax =
            "Numeric values : 12, 13.5, 0.7, 45.4f, -23";
        private final String likeSyntax =
            "like/notlike (String value using \'*\' as a wild card): "
                + "\"*flat*\", \"NTT*\"";

        // editing controls
        //private JTextField lhsText = null;
        private RecallingComboBox lhsText;
        private JTextField rhsText;
        private JComboBox operator;

        private boolean changesSaved = false;

        private String lhsTextBack;
        private String rhsTextBack;

        BinaryOpExprEditor(Frame owner, DefaultMutableTreeNode n) {
            super(owner, "Editing boolean expression");
            targetNode = n;
            init();
        }

        private void init() {
            BinaryKeywordExpr expr = null;
            expr = convertLeafToKeywordExpr(targetNode);
            if (expr == null) {
                expr =
                    new EqKeywordExpr(
                        new Keyword("KEYWORD", null),
                        new String("VALUE"));
            }

            getContentPane().setLayout(new BorderLayout());
            lhsTextBack = KeywordExpression.valueToString(expr.getLeftValue());
            rhsTextBack = KeywordExpression.valueToString(expr.getRightValue());
            
            //lhsText = new JTextField(lhsTextBack, 15);
            SharedKeywordList kwList 
                = SharedKeywordList.getListFromRegistry(SharedKeywordList.DefaultKeywordList);
            lhsText = new RecallingComboBox(new SharedListComboBoxModel(kwList));
            if (lhsTextBack.length() > 0) {
                kwList.add(lhsTextBack);
                lhsText.setSelectedItem(lhsTextBack);
            }
            lhsText.addActionListener(this);
            
            rhsText =
                new JTextField(
                    KeywordExpression.valueToString(expr.getRightValue()),
                    15);
            rhsText.addActionListener(this);

            operator = new JComboBox(operators);
            operator.setSelectedItem(expr.getOpString());
            operator.addActionListener(this);

            JPanel upperPanel = new JPanel();
            upperPanel.add(lhsText);
            upperPanel.add(operator);
            upperPanel.add(rhsText);

            getContentPane().add(upperPanel, "North");

            JPanel instructionPanel = new JPanel(new GridLayout(4, 1));
            JLabel instruction = new JLabel(keywordSyntax);
            instruction.setForeground(Color.black);
            instructionPanel.add(instruction);

            instruction = new JLabel(stringSyntax);
            instruction.setForeground(Color.black);
            instructionPanel.add(instruction);

            instruction = new JLabel(numberSyntax);
            instruction.setForeground(Color.black);
            instructionPanel.add(instruction);

            instruction = new JLabel(likeSyntax);
            instruction.setForeground(Color.black);
            instructionPanel.add(instruction);

            instructionPanel.setBorder(new EmptyBorder(5, 5, 5, 5));

            getContentPane().add(instructionPanel, "Center");

            okButton = new JButton("ok");
            okButton.addActionListener(this);

            cancelButton = new JButton("cancel");
            cancelButton.addActionListener(this);

            JPanel lowerPanel = new JPanel();
            lowerPanel.add(okButton);
            lowerPanel.add(cancelButton);
            getContentPane().add(lowerPanel, "South");
            lhsText.requestFocus();
        }

        public boolean okSelected() {
            return changesSaved;
        }

        // implementation of the ActionListener interface
        public void actionPerformed(ActionEvent a) {
            if (a.getSource() == okButton
                || a.getSource() == rhsText) {
                if (Keyword.validName(lhsText.getText())) {
                    if (isLegalRHS(rhsText.getText())) {
                        if (saveChanges()) {
                            setVisible(false);
                            dispose();
                        }
                    } else {
                        rhsText.setText(rhsTextBack);
                    }
                } else {
                    lhsText.setText(lhsTextBack);
                }
            } else if (a.getSource() == cancelButton) {
                setVisible(false);
                dispose();
            }
        }

        private boolean saveChanges() {
            String lhs = lhsText.getText();
            String rhs = rhsText.getText();
            if (KeywordExpression.stringToValue(lhs) == null) {
                userMessage("Value: " + lhs + " is invalid");
            } else if (KeywordExpression.stringToValue(rhs) == null) {
                userMessage("Value: " + rhs + " is invalid");
            } else if (operator.getSelectedItem() == null) {
                userMessage("Please select an operator");
            } else {
                try {
                    rhs = Float.valueOf(rhs).toString();
                } catch (NumberFormatException e) {
                    // too bad
                }
                targetNode.setUserObject(
                    new String(
                        lhs
                            + " "
                            + operator.getSelectedItem().toString()
                            + " "
                            + rhs));
                changesSaved = true;
            }
            return changesSaved;
        }
    }

    /**
     * Modal dialog for editing a BoolKeywordExpression.
     */

    class BoolExprEditor extends JDialog implements ActionListener {

        private DefaultMutableTreeNode targetNode = null;
        private JButton okButton = null;
        private JButton cancelButton = null;
        private final String[] operators =
            { AndKeywordExpr.opString, OrKeywordExpr.opString };

        // editing controls
        private JComboBox operator = null;

        private boolean changesSaved = false;

        BoolExprEditor(Frame owner, DefaultMutableTreeNode n) {
            super(owner, "Editing expression operator");
            targetNode = n;
            init();
        }

        private void init() {
            getContentPane().setLayout(new BorderLayout());
            operator = new JComboBox(operators);
            operator.setSelectedItem(targetNode.toString());
            operator.addActionListener(this);

            JPanel upperPanel = new JPanel();
            upperPanel.add(operator);

            getContentPane().add(upperPanel, "North");

            okButton = new JButton("ok");
            okButton.addActionListener(this);

            cancelButton = new JButton("cancel");
            cancelButton.addActionListener(this);

            JPanel lowerPanel = new JPanel();
            lowerPanel.add(okButton);
            lowerPanel.add(cancelButton);
            getContentPane().add(lowerPanel, "South");
        }

        public boolean okSelected() {
            return changesSaved;
        }

        // implementation of the ActionListener interface
        public void actionPerformed(ActionEvent a) {
            if (a.getSource() == okButton) {
                if (saveChanges()) {
                    setVisible(false);
                    dispose();
                }
            } else if (a.getSource() == cancelButton) {
                setVisible(false);
                dispose();
            }
        }

        private boolean saveChanges() {
            targetNode.setUserObject(operator.getSelectedItem());
            changesSaved = true;
            return changesSaved;
        }
    }

    /**
     * Returns the panelColor.
     * @return Color
     */
    public static Color getDefaultPanelColor() {
        return panelColor;
    }

    /**
     * Sets the panelColor.
     * @param panelColor The panelColor to set
     */
    public static void setDefaultPanelColor(Color panelColor) {
        ExpressionBuilder.panelColor = panelColor;
    }

}
