/**
 * @author M. Zamparelli
 * @version 1.0
 * A class for providing BaseNames
 */

package org.eso.gasgano.tools;

import gnu.regexp.RE;
import gnu.regexp.REException;
import gnu.regexp.REMatch;
import gnu.regexp.RESyntax;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.eso.gasgano.properties.PropertyDB;

public class BaseName {
    static RESyntax theSyntax = new RESyntax(RESyntax.RE_SYNTAX_PERL5);
    static String dateReg = "\\d{4}-\\d{2}-\\d{2}";
    static String timeReg = "\\d{2}:\\d{2}:\\d{2}.\\d{3}";
    static String regARCFILE =
        new String("^([A-Z]|[1-2])*\\." + dateReg + "T" + timeReg);
    // IMID_990429A_DIT=1.7726.fits
    // ISAAC.1999-10-09T10:47:50.048.fits
    private static final int SECONDS_AND_MILLI = 6;

    /**
     * @param fullPath The path of the file.
     * @return The DFS base name of the file, or the fullPath if there was
     * not valid base name found.
     */
    public static String extractBaseName(String fullPath) {
        String rc = fullPath;
        String tempBname = fullPath;

        // if no colon occurred the following might lead to OutOfBounds
        int endBname = tempBname.lastIndexOf(':') + SECONDS_AND_MILLI + 1;

        int startBname = tempBname.lastIndexOf(File.separatorChar) + 1;

        // assume that r. is the only prefix ever used...
        if (tempBname.startsWith("r.", startBname)) {
            // remove tha "r." pre-fix.
            startBname += 2;
        }
        if (endBname + 4 <= tempBname.length()
            && tempBname.substring(endBname, endBname + 4).equals("_tpl")) {
            endBname += 4;
        }
        try {
            tempBname = tempBname.substring(startBname, endBname);
        } catch (StringIndexOutOfBoundsException e) {
            if (PropertyDB.getInstance().isDebug(PropertyDB.debugBaseName)) {
                System.out.println(
                    "Parsing file name 1: <" + fullPath + "> " + e.toString());
            }
            tempBname = fullPath;
        }
        if (tempBname.equals(fullPath)) {
            endBname = fullPath.lastIndexOf('_');
            if (endBname == -1) {
                endBname = fullPath.lastIndexOf('.');
            }
            if (endBname == -1) {
                endBname = fullPath.length();
            }
            try {
                tempBname = tempBname.substring(startBname, endBname);
            } catch (StringIndexOutOfBoundsException e) {
                if (PropertyDB.getInstance().isDebug(PropertyDB.debugBaseName)) {
                    System.out.println("Parsing file name 2: " + e.toString());
                }
                tempBname = fullPath;
            }
        }
        rc = tempBname;
        if (PropertyDB.getInstance().isDebug(PropertyDB.debugBaseName)) {
            System.out.println("BASE:<" + rc + "> from <" + fullPath + ">");
        }
        return rc;
    }

    /**
     * Extract the product base name, use the default file 
     * separator character.
     */
    public static String extractProductBaseName(String fullPath) {
        return extractProductBaseName(fullPath, File.separatorChar);
    }

    /**
     * Extract the product base name using the specified file separator - 
     * this is useful for parsing path strings which contain file separators
     * that are of a specific platform (such as paths written in Reduction Block files.
     */
    public static String extractProductBaseName(
        String fullPath,
        char fileSeparator) {
        String rc = fullPath;
        String tempBname = fullPath;
        int endBname = tempBname.lastIndexOf(':') + 7;
        int startBname = tempBname.lastIndexOf(fileSeparator) + 1;
        // assume that r. is the only prefix ever used...
        // BIG ASSUMPTION!
        if (tempBname.startsWith("r.", startBname)) {
            // remove tha "r." pre-fix.
            startBname += 2;
        }
        // this is Paola's catch all suggestion
        endBname = fullPath.length();
        try {
            tempBname = tempBname.substring(startBname, endBname);
        } catch (StringIndexOutOfBoundsException e) {
            if (PropertyDB.getInstance().isDebug(PropertyDB.debugBaseName)) {
                System.out.println("Parsing file name 2: " + e.toString());
            }
            tempBname = fullPath;
        }

        rc = tempBname;
        if (PropertyDB.getInstance().isDebug(PropertyDB.debugBaseName)) {
            System.out.println("BASE:<" + rc + "> from <" + fullPath + ">");
        }
        return rc;
    }

    public static String extractTest(String input) {
        REMatch theMatch = null;
        // RESyntax theSyntax  = new RESyntax(RESyntax.RE_SYNTAX_PERL5);

        RE exp = null, initExp = null;
        String reg = new String("[^/]*$");
        // set up all needed expressions (should be done in <init>)
        try {
            exp = new RE(regARCFILE);
            initExp = new RE(reg);
        } catch (REException ree) {
        }

        theMatch = initExp.getMatch(input);

        if (theMatch != null) {
            // System.out.println("basename:  " + theMatch.toString());
            input = theMatch.toString();
            // get rid of prefix
            if (input.startsWith("r.")) {
                input = input.substring(2);
            }

            // get rid of all possible suffices, which should be taken from the preferences
            if (input.endsWith(".fits")) {
                input = input.substring(0, input.length() - 5);
            }
            if (input.endsWith(".tfits")) {
                input = input.substring(0, input.length() - 6);
            }
            if (input.endsWith(".rb")
                || input.endsWith(".rX")
                || input.endsWith(".rY")
                || input.endsWith(".rZ")) {
                input = input.substring(0, input.length() - 2);
            }

            if (exp.isMatch(input)) {
                System.out.println("<matches ARCFILE>");
            }
            return input;
        }
        // exception occured
        return "FAILED";
    }

    private static void check(REMatch m, String expect, int x) {
        if ((m == null) || !m.toString().equals(expect))
            System.out.print("Failed");
        else
            System.out.print("Passed");
        System.out.println(" test #" + x);
    }

    public static void main(String argv[]) {
        String input = argv[0];
        File aFile = null;
        if (input == null)
            return;
        aFile = new File(input);
        if (!aFile.exists()) {
            //
            System.out.println(
                "** This test uses methods with REGEXP *NOT* used by Gasgano ** ");
            System.out.println("Test Basename: " + extractTest(input));
            System.out.println("== (for comparison)  ==");
            System.out.println("BaseName: " + extractBaseName(input));
            System.out.println(
                "ProductBaseName: " + extractProductBaseName(input));
            System.out.println("\n");
        } else {
            FileReader dataSource = null;
            try {
                dataSource = new FileReader(input);
            } catch (FileNotFoundException ex) {
                System.out.println("File not found: " + input);
            }
            if (dataSource == null) {
                System.exit(0);
            }
            StringBuffer buff = new StringBuffer();
            BufferedReader br = new BufferedReader(dataSource);
            try {
                while (true) {
                    String s = br.readLine();
                    if (s == null) {
                        break;
                    } else {
                        if (s.startsWith("PRODUCT")) {
                            System.out.println(
                                "** This test uses the same methods currently used by Gasgano ** ");
                            String product = s.substring(9, s.length());
                            System.out.println(
                                "product: "
                                    + product
                                    + " \tbasename: "
                                    + extractProductBaseName(product));
                        } else {
                            System.out.println(
                                "to match: "
                                    + s
                                    + " \tbasename:"
                                    + extractBaseName(s));
                        }
                    }
                }
            } catch (IOException e) {
                // error reading from the Reader.
                System.out.println("failure");
            }
        }
    }
}
