package org.eso.ohs.core.dbb.rmi;

// $Id: DbbLocalEngine.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryChunks;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/**   
 * This is the main engine of a database browser: builds SQL
 * statements, performs queries (within threads), returns result data.
 *
 * @author Bob Kemp
 * @author C. Boarotto, ESO 10-Sep-2000
 *
 */
public class DbbLocalEngine implements DbbSqlEngine {

private static final String rcsid = "$Id: DbbLocalEngine.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $";
private static Logger stdlog_ = 
    Logger.getLogger( DbbLocalEngine.class );

    /** Maximum number of retries when contacting the server */
    protected static final int MAX_RETRIES = 10;

    /** Server url */
    protected final String SERVER_URL;

    /** Retries counter */
    protected int retryCount_ = 0;

    /** Reference to the remote sql engine */
    protected DbbRemoteEngine rmiMgr_;

  
    //----------------------------------------
    // Constructor
    //----------------------------------------

    /**
     * Create a DbbLocalEngine object setting the server url.
     * @param url Server url
     */
    public DbbLocalEngine(String url) {
	SERVER_URL = url;
    }


    //----------------------------------------
    // Private methods
    //----------------------------------------

    /** 
     * Make sure a connection is ok.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server    
     */
    private void checkConnection()
	throws RemoteException,
	SQLException, 
	MalformedURLException, NotBoundException {

	if (rmiMgr_ == null) {
	    resetServerConnection(null);
	}
    }


    /** 
     * Rest the server connection
     *
     * @param e Remote exception
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server    
     */
    private void resetServerConnection(RemoteException e)
	throws RemoteException,
	SQLException, 
	MalformedURLException, NotBoundException {

	logCompletedTransaction("Reset server connection: begin");

	if (++retryCount_ > MAX_RETRIES) {
	    retryCount_ = 0;
	    throw e;
	}
	if (e != null) stdlog_.warn("",e);

	try {
	    // First try to get a new StorageManager instance.
	    rmiMgr_ = factory();
	}
	catch (RemoteException exc) {
	    // Lost contact with server?  Try reconnecting.
	    rmiMgr_ = factory();
	}

	logCompletedTransaction("Reset server connection: done");
    }


    /**
     * Log a transaction.
     * @param caller Caller
     */
    private void logCompletedTransaction(String caller) {
	if(stdlog_.isDebugEnabled())
	    stdlog_.debug("Completed call to " + caller);
    }


    //----------------------------------------
    // Protected methods
    //----------------------------------------

    /** 
     * Get remote engine. It is protected because it might be
     * redefined in a subclass.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server */
    protected DbbRemoteEngine factory()
	throws RemoteException,
	SQLException,
	MalformedURLException,
	NotBoundException {

	logCompletedTransaction("Server url: "+SERVER_URL);
	return ((DbbFactory)Naming.lookup(SERVER_URL)).getEngine();
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Create and start a thread to get the result set and to process it
     * into an Object array.  
     *
     * @param chunks	Sql query chunks
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public DbbQuery startQuery(DbbSqlQueryChunks chunks)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException {
	for ( ; ; ) {
	    try {
		checkConnection();
		DbbQuery result = new DbbLocalQuery(rmiMgr_.startQuery(chunks));
		logCompletedTransaction("startQuery");
		return result;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }


    /**
     * Number of records in a result set   
     *
     * @param chunks	Sql query chunks
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public int execSqlCount(DbbSqlQueryChunks chunks)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException {

	for ( ; ; ) {
	    try {
		checkConnection();
		int result = rmiMgr_.execSqlCount(chunks);
		logCompletedTransaction("execSqlCount");
		return result;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }

 
    /**
     * Add default colums to a particular chunk
     *
     * @param chunks	Sql query chunks
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addDefaultColumnsChunk(DbbSqlChunk chunk)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException {

	for ( ; ; ) {
	    try {
		checkConnection();
		rmiMgr_.addDefaultColumnsChunk(chunk);
		logCompletedTransaction("addDefaultColumnsChunk");
		return;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }


    public DbbSqlChunk[] getDefaultColumnsChunk() 
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException {

	for ( ; ; ) {
	    try {
		checkConnection();
		DbbSqlChunk[] chunks = rmiMgr_.getDefaultColumnsChunk();
		logCompletedTransaction("addDefaultColumnsChunk");
		return chunks;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }

    /**
     * Add a preferential join condition to be considered when building
     * automatically the query.
     *
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_col	String indicating the primary key table column.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_col	String indicating the foreign key table column.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addJoinCondition(DbbSqlTable from_tbl,
				 String from_col,
				 DbbSqlTable to_tbl,
				 String to_col)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException {

	for ( ; ; ) {
	    try {
		checkConnection();
		rmiMgr_.addJoinCondition(from_tbl, from_col,
					 to_tbl, to_col);

		logCompletedTransaction("addJoinCondition");
		return;
	    }
	    catch (RemoteException e) {
		resetServerConnection(e);
	    }
	}
    }
    
    
    /**
     * Add left outer join condition to be considered when building
     * automatically the query.
     *
     * @param from_tbl  DbbSqlTable indicating the primary key table.
     * @param from_col  String indicating the primary key table column.
     * @param to_tbl    DbbSqlTable indicating the foreign key table.
     * @param to_col    String indicating the foreign key table column.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addCompositeJoinCondition(DbbSqlTable from_tbl,
                 String from_cols[],
                 DbbSqlTable to_tbl,
                 String to_cols[], boolean leftOuterJoin)
    throws SQLException, RemoteException,
    MalformedURLException, NotBoundException {

    for ( ; ; ) {
        try {
        checkConnection();
        rmiMgr_.addCompositeJoinCondition(from_tbl, from_cols,
                     to_tbl, to_cols, leftOuterJoin);

        logCompletedTransaction("addLeftOuterJoinCondition");
        return;
        }
        catch (RemoteException e) {
        resetServerConnection(e);
        }
    }
    }
}



