package org.eso.ohs.dfs;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.NotActiveException;
import java.io.ObjectInputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;

/**
 * Simple implementation of the StorableObject interface.
 * 
 * <p>
 * $Id: SimpleStorableObject.java,v 1.31 2005/10/07 08:43:06 ddorigo Exp $
 * 
 * @author cmavros 2002/JUNE/03 [1766/P2PP] If name field is blank assign the
 *         DEFAULT_NAME to value.
 */
public abstract class SimpleStorableObject implements StorableObject {

    /** Standard log channel */
    protected static Logger stdlog_ = Logger
            .getLogger(SimpleStorableObject.class);

    /** Version id for object serialization */
    static final long serialVersionUID = 7689080168921994553L;

    /**
     * The objects unique id: if +ve, then it is also the database id
     */
    private long id_;

    /** Initial value of the Name property */
    public final static String DEFAULT_NAME = "No Name";

    /** Support for handling PropertyChangeEvent */
    protected transient EventListenerList listenerList;

    /** PropertyChangeEvent to pass to listeners */
    private transient PropertyChangeEvent propertyChangeEvent;

    /**
     * User assigned name of this object
     */
    private String name_ = DEFAULT_NAME;

    /**
     * <br>
     * ChangeLog: A M Chavan, 09-Aug-2001: changed to private
     */
    private transient boolean supressPropertyChangeEvents_ = false;

    protected transient PropertyChangeEvent templateEvent_;

    protected transient PropertyChangeEvent findingChartEvent;

    private boolean isNew_ = false;

    /**
     * Default constructor. Initialise property change event support.
     */
    public SimpleStorableObject() {
        initialise();
    }

    /**
     * Ignore all requests to fire a PropertyChangeEvent.
     * 
     * @see #resumePropertyChangeEvents() <br>
     *      ChangeLog: A M Chavan, 09-Aug-2001
     */
    protected void suspendPropertyChangeEvents() {
        supressPropertyChangeEvents_ = true;
    }

    /**
     * Ignore all requests to fire a PropertyChangeEvent.
     * 
     * @see #suspendPropertyChangeEvents() <br>
     *      ChangeLog: A M Chavan, 09-Aug-2001
     */
    protected void resumePropertyChangeEvents() {
        supressPropertyChangeEvents_ = false;
    }

    /**
     * Deserialize a SimpleStorableObject and initialise property change event
     * support.
     * 
     * @exception IOException
     *                an I/O error occurred while reading the object from
     *                external media.
     * @exception ClassNotFoundException
     *                The class for a serialized business object taken from disk
     *                cannot be found. This should never happen.
     * @exception NotActiveException
     *                thrown if the ObjectInputStream is used when not active.
     */
    private void readObject(ObjectInputStream in)
            throws IOException, ClassNotFoundException, NotActiveException {
        in.defaultReadObject();
        initialise();
    }

    /**
     * Initialise property change event support.
     */
    private void initialise() {

        listenerList = new EventListenerList();
        propertyChangeEvent = new PropertyChangeEvent(this, null, null, null);
        findingChartEvent = new PropertyChangeEvent(this, "FindingCharts",
                null, null);
        templateEvent_ = new PropertyChangeEvent(this, "Template", null, null);

    }

    /**
     * Set new value for id_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setId(long newValue) {
        id_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for id_ property
     * 
     * @return the current value of the property
     */
    public long getId() {
        return id_;
    }

    /**
     * Set new value for name_ property
     * 
     * @param newValue
     *            the new value for the property
     */
    public void setName(String newValue) {
        if (newValue == null || newValue.trim().length() == 0) {
            name_ = DEFAULT_NAME;
        } else {
            name_ = newValue;
        }
        firePropertyChangeEvent();
    }

    /**
     * Get value for name_ property
     * 
     * @return the current value of the property
     */
    public String getName() {
        return name_;
    }

    /**
     * Returns the string used to refer to the given type of BusinessObject in
     * messages to the user.
     * 
     * @param cls
     *            Class object for required BusinessObject
     * @return user-oriented name
     */
    public static String getDisplayName(Class cls) {
        return (String) getAttributeValue(cls, "getDisplayName");
    }

    /**
     * Returns the filename suffix for files containing the given type of
     * object.
     * 
     * @param cls
     *            Class object for required BusinessObject
     * 
     * @return suffix for filename when stored in local filesystem
     */
    public static String getSuffix(Class cls) {
        return (String) getAttributeValue(cls, "getSuffix");
    }

    /**
     * Returns the value of the specified static zero-arguments method of the
     * specified class.
     * 
     * @param cls
     *            Class whose static method is to be called.
     * @param methName
     *            Name of the static method is to be called.
     * 
     * @return result of the method
     */
    private static Object getAttributeValue(Class cls, String methName) {
        if (!SimpleStorableObject.class.isAssignableFrom(cls)) {
            throw new IllegalArgumentException(
                    "Internal error: not a SimpleStorableObject subclass");
        }

        try {
            Method getter = cls.getMethod(methName, new Class[] {});

            return getter.invoke(null, new Class[] {});
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.getTargetException().printStackTrace();
        } catch (NullPointerException e) {
            stdlog_.debug("Class  = " + cls);
            stdlog_.debug("Method = " + methName);
            e.printStackTrace();
            throw e;
        }

        return null;
    }

    /**
     * Returns a string representation of the object's arguments. Mainly used
     * for debugging.
     * 
     * @return the string representation
     */
    public String paramString() {
        return "name='" + name_ + "'" + ",id=" + id_;
    }

    /**
     * Returns a string representation of the object. Mainly used for debugging.
     * 
     * @return the string representation
     */
    public String toString() {
        String className = getClass().getName();

        return className.substring(className.lastIndexOf('.') + 1) + "["
                + paramString() + "]";
    }

    /**
     * Adds a new listener to be notified of changes.
     * 
     * @param l
     *            the listener
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        listenerList.add(PropertyChangeListener.class, l);
    }

    /**
     * Removes a new listener, which will no longer be notified of changes.
     * 
     * @param l
     *            the listener
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        listenerList.remove(PropertyChangeListener.class, l);
    }

    /**
     * Informs listeners of a property change. No details of which property of
     * the old/new value are given.
     */
    protected void firePropertyChangeEvent() {
        if (!supressPropertyChangeEvents_) {
            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                if (listeners[i] == PropertyChangeListener.class) {
                    ((PropertyChangeListener) listeners[i + 1])
                            .propertyChange(propertyChangeEvent);
                }
            }
        }
    }

    protected void firePropertyChangeEvent(PropertyChangeEvent ev) {
        if (!supressPropertyChangeEvents_) {
            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                if (listeners[i] == PropertyChangeListener.class) {
                    ((PropertyChangeListener) listeners[i + 1])
                            .propertyChange(ev);
                }
            }
        }
    }

    /**
     * Returns the isNew_.
     * 
     * @return boolean
     */
    public boolean isNew() {
        return isNew_;
    }

    /**
     * Sets the isNew.
     * 
     * @param isNew_
     *            The isNew_ to set
     */
    public void setIsNew(boolean isNew) {
        isNew_ = isNew;
    }

    public boolean isReadOnly() {
        return false;
    }

}
