package org.eso.ohs.phase2.visibility;

/**
 * FLI class calculates the Fractional Lunar Illumination. 
 *
 * There isn't a single SLALIB routine that calculates 
 * Fractional Lunar Illumination.  You can do it to a good 
 * approximation by calling slaRdplan twice, for the Sun and Moon 
 * respectively, and then calling slaDsep.  This gives you how 
 * far apart the Sun and Moon are in the sky.  Call it theta
 * and get the fractional illumination using (1-cos(theta))/2.
 *
 * @author C. Mavros for ESO
 * @version 25 July 2001
 */

import org.apache.log4j.Logger;

public class FLI extends JNIWrapper{

   /**
    * Standard log channel
    * <br>ChangeLog: cmavros 21-12-01 added
    */
   private static Logger stdlog_ =
   Logger.getLogger( FLI.class );

    /**
    * The constant the SLALIB function uses 
    * for the Sun and the Moon
    */
    private static final int SUN = 0;
    private static final int MOON = 3;

    /**
    * Position in RA and DEC position in the
    * array returned from the SLALIB function
    */
    private static final int MOON_RA = 0;
    private static final int MOON_DEC = 1;
    private static final int SUN_RA = 0;
    private static final int SUN_DEC = 1;


   /**
    * getFLI calls the following SLALIB functions: 
    * slaRdplan - Sun position
    * slaRdplan - Moon position
    * slaDsep - result theta (how far apart the Sun and Moon are in the sky)
    * FLI = (1-cos(theta))/2.
    *
    * @param jDate Julian Date of OB
    * @param longitude of observatory
    * @param lat of observatory
    * @param boolean considerHorizon if true and moon is below the horizon
    *       return -1.0 instead of the FLI. 
    * @return fli
    */
 
    public static double getFLI(double jDate, double longitude, double phi,
            boolean considerHorizon ){

        double rdpMoon[] = new double[3];
        double rdpSun[] = new double[3];
        double theta = 0.0;
        double fli = 0.0;

        /** Call SLALIB function slaRdplan twice,
         * First for the Moon position and then the Sun position
         * Returning the RA and Dec of the moon and sun in radians.
         */
        rdpMoon = jnislaRdplan((jDate- 2400000.5), MOON, longitude, phi);
        rdpSun = jnislaRdplan((jDate- 2400000.5), SUN, longitude, phi);

        //stdlog_.info.out.println ("visiplot.SunAngle.getFLI sunRA "+ rdpSun[SUN_RA]);
        //stdlog_.info.out.println ("visiplot.SunAngle.getFLI sunDEC "+ rdpSun[SUN_DEC]);


        /** Pass the previously calculated RA and Dec of the Moon 
         * and the Sun to the SLALIB function slaDsep to 
         * calculate the distance between the Moon and the Sun 
         */
        theta = jnislaDsep(rdpMoon[MOON_RA], rdpMoon[MOON_DEC], 
                             rdpSun[SUN_RA], rdpSun[SUN_DEC]);

        fli = (1.0 - Math.cos(theta))/2;
        //stdlog_.inf("visiplot.SunAngle.getFLI fli "+ fli);

        return(fli);
    }
}
