package org.eso.ohs.scripting;

/**
 * The ScriptInterpreter interface must be implemented when adding new
 * script interpreters to the system. A static initializer in the 
 * ScriptInterpreter sub-class will initialize the object and, upon 
 * successful initialization, register the ScriptInterpreter with the 
 * ScriptInterpreterManager.
 *
 * The code below shows example static initialization code for the 
 * JaclScriptInterpreter. The code only registers the JaclScriptInterpreter 
 * upon successful instantiation of the tcl.lang.Interp class. 
 * The model should be followed for all ScriptInterpreter implementations.
 *
 *     class JaclScriptInterpreter implements ScriptInterpreter {
 *         private static Interp interpreter = null;
 *         private static JaclScriptInterpreter instance;
 *         static {
 *             interpreter = new Interp(); // tcl.lang.Interp()
 *             instance = new JaclScriptInterpreter();
 *             ScriptInterpreterManager.getInstance()
 *                     .registerScriptInterpreter(instance);
 *         }
 *         //... rest of interface implementation.
 *     }
 *
 * @author Nick Kornweibel
 */

import java.io.File;

public interface ScriptInterpreter {

    /**
     * Execute the script text passed
     * @param script The text to be executed
     */
    public void executeScript(String script) throws ScriptInterpreterException;

    /**
     * Execute the text in the file given
     * @param scriptPath The path to the text file
     */
    public void executeFile(File scriptPath) throws ScriptInterpreterException;

    /**
     * Get the names of the languages supported by this interpreter
     * @return Array of String objects of the supported language names
     */
    public String[] getLanguageNames();

    /**
     * Get the file extensions supported by this interpreter
     * @return Array of String objects of the supported file extensions
     */
    public String[] getFileExtensions();
}
