package org.eso.ohs.scripting;

/**
 * The ScriptingUtilities class provides a utilities library for use by 
 * script writers. Such functions as PAF data tokenizing and RA conversions
 * are provided in this class.
 * The ScriptingUtilities object is returned by the getUtils() call in
 * ScriptingBox interface.
 * @author Nick Kornweibel
 */

import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.PAFTable;

public class ScriptingUtilities {

    /**
     * Get the milli arcseconds from the RA String
     * @param String ra value e.g. "05:45:36.5"
     * @return long milli arc seconds RA value
     */

    public long getRAMilliarcseconds(String ra) {
        long rc = -1L;
        try {
            rc = (long)Convert.HHMMSSToMilliarcsec(ra);
        } catch (IllegalArgumentException ex) {
            rc = -1L;
        }
        return rc;
    }

    /**
     * Get the RA Coordinate string from milli arcseconds
     * @param long milli arc seconds RA value
     * @return String ra value e.g. "05:45:36.5"
     */

    public String getRACoordinate(long ra) {
        String rc;
        try {
            rc = Convert.milliarcsecToHHMMSS((int)ra);
        } catch (IllegalArgumentException ex) {
            rc = new String();
        }
        return rc;
    }

    /**
     * Get the milli arcseconds from the DEC String
     * @param String dec value e.g. "-78:11:09.0"
     * @return long milli arc seconds declination value
     */

    public long getDecMilliarcseconds(String dec) {
        long rc = -1L;
        try {
            rc = (long)Convert.DDMMSSToMilliarcsec(dec);
        } catch (IllegalArgumentException ex) {
            rc = -1L;
        }
        return rc;
    }

    /**
     * Get the Dec Coordinate string from milli arcseconds
     * @param long milli arc seconds declination value
     * @return String dec value e.g. "-78:11:09.0"
     */

    public String getDecCoordinate(long dec) {
        String rc;
        try {
            rc = Convert.milliarcsecToDDMMSS((int)dec);
        } catch (IllegalArgumentException ex) {
            rc = new String();
        }
        return rc;
    }


    /**
     * Get a PAFTable object built from the PAF format string
     * returns an empty PAFTable if string is not PAF format.
     * @param String PAF format string
     * @return PAFTable built from input string
     */

    public PAFTable parsePAF(String pafData) {
        PAFTable rc = new PAFTable();
        rc.loadFromString(pafData);
        return rc;
    }

    /**
     * Return an empty string if instead of null
     * @param String input
     * @return empty string if input null, else input
     */

    public static String nullToEmptyString(String in) {
        String rc = in;
        if (rc == null) {
            rc = "";
        }
       return rc;
    }
}
